# This controls "import *" and actually imports only what is defined here
__all__ = ["ssh_host", "ssh_port", "creds_path", "global_config", "local_config", "provider"]

# These should not be modified here!
ssh_host = ""
ssh_port = ""
creds_path = ""
global_config = ""
local_config = ""
provider = None


from os import name
import cloudsync as cs
import tempfile as tf
from .config import make_auth
import pandas as pd
import json
from . import vars

def connect():
    if vars.creds_path == "":
        make_auth()
    oauth_config = cs.command.utils.generic_oauth_config('onedrive')
    vars.provider = cs.create_provider('onedrive', oauth_config=oauth_config)
    f = open(vars.creds_path, 'r')
    creds = json.load(f)
    vars.provider.connect(creds)

def change_namespace(path_in_ns, namespace=None):
    # Change namespace to shared folder
    ns = vars.provider.list_ns()
    if namespace is None:
        shared_folder_name = path_in_ns
        for x in ns:
            if shared_folder_name in x.name:
                break
        print("Changing namespace to: {}".format(x.id))
        vars.provider.namespace = x
    else:
        for x in ns:
            if namespace in x.name:
                break
        print("Changing namespace to: {}".format(x.id))
        vars.provider.namespace = x

def download_file(filename, namespace=None, output=None):
    if vars.provider is None:
        connect()
    if not namespace is None:
        change_namespace(namespace)
    if output is None:
        tmp = tf.NamedTemporaryFile()
        print("Downloading to {} ...".format(tmp.name))
    else:
        tmp = open(output, 'w')
    vars.provider.download_path(filename, tmp)
    tmp.seek(0) # Go back to first line
    return tmp

def upload_file():
    print("> upload_file()")
    print("Still to be implemented...")


import os
import subprocess
import tempfile as tf
from . import vars

def remote_exec(path, rdir="", verbose=True, logfile=None):
    # If localhost, return
    if 'localhost' in vars.ssh_host or '127.0.0.1' in vars.ssh_host:
        print("Running on local machine...")
        return
    print("Running from file: {}".format(path))
    # Open the calling script (from path) and read the file
    fin = open(path, 'r')
    # Split the script and take everything after separator
    s = fin.read().split("remote_exec(__file__)")[-1]
    # Do we need to import CloudComputing? 
    if "CloudComputing" in s or "cc" in s:
        s = "import CloudComputing as cc\n" + s
    # Write to file
    tmp = os.path.join(tf.gettempdir(), os.urandom(8).hex() + '.py')
    fout = open(tmp, 'w')
    fout.write(s)
    fout.close()
    # CD to remote dir (if any)
    cmd = cmd = "/usr/bin/ssh -p {} {}".format(vars.ssh_port, vars.ssh_host)
    cmd = cmd + " 'cd {} &&".format(rdir)
    # Run over SSH
    print(vars.ssh_host + " " + vars.ssh_port)
    cmd = cmd + " python -V && python -u -' < {}".format(tmp)
    print(cmd)
    if not verbose:
        cmd = cmd + " 1>/dev/null 2>&1"
    if not logfile is None:
        print("Logging to file: {}".format(logfile))
        cmd = cmd + " > {}".format(logfile)
    subprocess.Popen(cmd, shell=True)
    # Remove the temp file
    os.system("rm {}".format(tmp))
    # Exit to prevent the calling script to run locally
    exit(0)


import cloudsync as cs
import json
import os
from os.path import exists
import subprocess
from . import vars
import configparser

'''      ------------------      Cloud Storage     ------------------      '''
def get_auth_token():
    oauth_config = cs.command.utils.generic_oauth_config('onedrive')
    provider = cs.create_provider('onedrive', oauth_config=oauth_config)
    creds = provider.authenticate()
    vars.provider = provider
    return creds

def save_auth_token(fname="$HOME/.cc_auth.json", creds=None):
    f = open(fname, 'w')
    json.dump(creds, f)

def check_auth(silent=False):
    defaultPath = os.environ['HOME'] + "/.cc_auth.json"
    if exists(defaultPath):
        if not silent:
            print("[INFO] Configuration found in: {}".format(defaultPath))
        return defaultPath
    else:
        if not silent:
            print("[INFO] No configuration found in default path.")
        return False

def make_auth(Force=False):
    # cloud_storage
    if check_auth() == False:
        print("Starting OneDrive auth...")
    else:
        if Force == False:
            print("Use check_auth(Force=True) to overwrite the existing OAuth token.")
            return
    token = get_auth_token()
    defaultPath = os.environ['HOME'] + "/"
    print("Specify path to save OneDrive auth token ({}): ".format(defaultPath), end='')
    f = input() or defaultPath
    f = f + ".cc_auth.json"
    save_auth_token(f, creds=token)
    print("[INFO] Token saved to file: {}".format(f))



'''      ------------------      Remote   Exec     ------------------      '''
def check_config(silent=False):
    defaultPath = os.environ['HOME'] + "/." + os.environ['USER'] + "-config.ini"
    if exists(defaultPath):
        if not silent:
            print("[INFO] Global SSH configuration found in {}".format(defaultPath))
        vars.global_config = defaultPath
    else:
        if not silent:
            print("[WARNING] Global SSH configuration not found. Call config.make_config() to create one.")

def make_config(local=False):
    if local:
        defaultPath = "./config.ini"
    else:
        defaultPath = os.environ['HOME'] + "/." + os.environ['USER'] + "-config.ini"
        if exists(defaultPath):
            print("[INFO] Global configuration already set. This will overwrite the existing configuration.")
    host = input("SSH user@host: ")
    if not "@" in host:
        print("Please provide user and host (e.g., admin@127.0.0.1")
        return
    port = input("SSH port (22): ") or "22" # Default: 22
    f = open(defaultPath, 'w')
    if f.closed:
        print("[ERROR] Unable to open file {}".format(defaultPath))
        return
    f.write("[SSH]\n")
    f.write("host = {}\n".format(host))
    f.write("port = {}\n".format(port))
    f.close()
    print("[INFO] {} configuration saved successfully.".format("Local" if local else "Global"))
    if local:
        vars.local_config = defaultPath
    else:
        vars.global_config = defaultPath

def load_config():
    p = os.environ['HOME'] + "/." + os.environ['USER'] + "-config.ini"
    if exists(p):
        vars.global_config = p
    p = "./config.ini"
    if exists(p):
        vars.local_config = p
    if vars.global_config == "" and vars.local_config == "":
        print("[WARNING] You need to set either a local or global configuration file.")
    c = configparser.ConfigParser()
    if vars.local_config == "":
        c.read(vars.global_config)
    else:
        c.read(vars.local_config)
    return c

def check_ssh_connection():
    if vars.global_config == "" and vars.local_config == "":
        print("[ERROR] No valid configuration file found. Exiting...")
    cmd = "ssh -p {} {} uname -n".format(vars.ssh_port, vars.ssh_host)
    print("Testing connection: " + cmd)
    cmd = "ssh -p {} {} uname -n".format(vars.ssh_port, vars.ssh_host)
    try:
        out = subprocess.check_output(cmd, shell=True).decode().strip()
    except subprocess.CalledProcessError:
        print("[WARNING] SSH connection issue.")
        return
    print("[INFO] Connected to host: {}".format(out))


from .cloud_storage import connect, change_namespace, download_file
from .config import get_auth_token, save_auth_token, check_auth, make_auth, check_config, make_config, load_config, check_ssh_connection
from .remote_exec import remote_exec
from . import vars

# CloudComputing version
__version__ = "0.0.8"
# Author (GitHub username)
__author__ = "mp1994"

## Global variables
# cloud_storage
vars.creds_path = check_auth(silent=True)
creds = vars.creds_path
# remote_exec
check_config(silent=True)
c = load_config()
vars.ssh_host = c['SSH']['host']
vars.ssh_port = c['SSH']['port']


from setuptools import setup, find_packages

VERSION = '0.0.8' 
DESCRIPTION = 'CloudComputing package'
LONG_DESCRIPTION = 'The CloudComputing package can be used to ease remote executing over SSH and cloud storage (OneDrive) with Python.'

# Setting up
setup(
       # the name must match the folder name
        name="CloudComputing", 
        version=VERSION,
        author="Mattia Pesenti",
        author_email="<mattia.pesenti@gmail.com>",
        url="https://github.com/mp1994/CloudComputing",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=['cloudsync', 'cloudsync-onedrive'], # add any additional packages that 
        # needs to be installed
        
        keywords=['python', 'cloud computing', 'onedrive'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Developers",
            "Programming Language :: Python :: 3",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: POSIX :: Linux",
        ]
)